import numpy as np
from ConfigureReader import *
from ImageReader import *
from MainDisplay import *

# Xu Minghao
projector_image_width = getProjectorImageWidth()
projector_distance = getProjectorDistance()
gamma = getGamma()
np_left = getImage('src/Left.jpg')
np_right = getImage('src/Right.jpg')
# calculate overlap pixels
overlap_pixels = round(getImageWidth(np_left)*(projector_image_width - projector_distance)/projector_image_width)


# Vaibhav
def getDisplayImage(np_tuple):
    # to RGB
    blended_left = np.rint(np_tuple[0]*255).astype(np.uint8)
    blended_right = np.rint(np_tuple[1]*255).astype(np.uint8)
    # save images
    cv.imwrite('src/blended_left.jpg', blended_left)
    cv.imwrite('src/blended_right.jpg', blended_right)
    return blended_left, blended_right

# Xu Minghao
def intensityController(distance, monotonicity):

    if monotonicity == 'increase':
        return -(1/overlap_pixels)*distance + 1
    elif monotonicity == 'decrease':
        return (1/overlap_pixels)*distance
    # a non-linear approach: -2*np.power(distance/overlap_pixels, 2)+1

# Xu Minghao
def maskGenerator(mask, direction):

    for columns in range(getImageWidth(mask)-overlap_pixels-1, getImageWidth(mask)):
        mask[0, columns, :] *= intensityController(
            columns-(getImageWidth(mask)-overlap_pixels-1), monotonicity='increase')
    # replacement is more efficient than multiplication
    for rows in range(1, getImageHeight(mask)):
        mask[rows, :, :] = mask[0, :, :]

    if direction == 'left':
        return mask
    elif direction == 'right':
        # flip direction of mask to right
        return np.flip(mask, axis=1)


# Xu Minghao, Vaibhav
def gammaCorrection(img, direction, gamma):

    if direction == 'left':
        for rows in range(getImageHeight(img)):
            for columns in range(getImageWidth(img)-overlap_pixels-1, getImageWidth(img)):
                img[rows][columns][:] *= np.power(intensityController(
                    columns-(getImageWidth(img)-overlap_pixels-1), monotonicity='increase'), gamma)
        return img

    elif direction == 'right':
        for rows in range(getImageHeight(img)):
            for columns in range(overlap_pixels-1):
                img[rows][columns][:] *= np.power(
                    intensityController(columns, monotonicity='decrease'), gamma)
        return img

# Shiva
def calculate(np_left, np_right):
    # generate masks
    mask_left = maskGenerator(
        np.ones(np_left.shape), direction='left')
    mask_right = maskGenerator(
        np.ones(np_right.shape), direction='right')
    # blend images with masks
    with_mask_left = np.multiply(np_left, mask_left)
    with_mask_right = np.multiply(np_right, mask_right)
    # apply gamma correction
    blended_left = gammaCorrection(with_mask_left, direction='left', gamma=gamma)
    blended_right = gammaCorrection(with_mask_right, direction='right', gamma=gamma)
    return blended_left, blended_right
